/*
*********************************************************************************************************
*
*             init/control code for peripharls like temperature, light, accelerometer and compass sensor
*                                     
*                                           Renesas RL78
*                                               on the
*                                     YRDKRL78G13 Evaluation Board
*
* Filename      : peripherals.c
* Version       : V1.00
*********************************************************************************************************
*/

/*
*******************************************************************************
** Include files
*******************************************************************************
*/
#include <includes.h>
#include <string.h>
#include <nmeap.h>
#include <CG_macrodriver.h> 
#include <CG_userdefine.h>
#include <CG_serial.h>
#include <peripherals.h>
#include <calibration.h>
#include <monitor.h>

/*
*******************************************************************************
**  Locals
*******************************************************************************
*/
static UCHAR nmea_cmd[NMEAP_MAX_SENTENCE_LENGTH];             /* buffer that contains NMEA strings */ 

/*
*******************************************************************************
** Globeals
*******************************************************************************
*/
GpsLogData gpsLogData = {0,0,0,0,0,0};                        /* holds GPS data to be stored to SD card */

/*
*******************************************************************************
**  Externals
*******************************************************************************
*/
extern nmeap_gga_t              gga;		          /* this is where the data from GGA messages will show up */
extern nmeap_rmc_t              rmc;		          /* this is where the data from RMC messages will show up */
extern int NMEA_Init(void);
extern void pushNmeaBuff(char *buffer, unsigned char length);
extern void ApproximateSolarCoordinates(DateTime *timeStamp, float lat, float lon, float *azimuth, float *elevation);


/*
*********************************************************************************************************
*                                          initPeripherals
*
* Description : initializes all peripherals that need initialization
*
* Arguments   : none
*
* Returns     : none
*
* Notes       : none
*********************************************************************************************************
*/
void initPeripherals()
{
     unsigned char i2cbuf[4];              
     
    IICA0_Init();                   /* initializes the IICA0 module. */  
    SAU1_Init();                    /* init serial array unit 1 */
    NMEA_Init();                    /* init NMEA parser */

   
    UART2_Start();                  /* init/start UART 2 -> GPS module */        
    
      /* Accelerometer init */
     i2cbuf[0] = ACCEL_REG_PWRCTL;
     i2cbuf[1] = ACCEL_CTL_MEASURE;
     IICA0_MasterSendStart(ACCEL_ADDR & ~(0x01), i2cbuf, 2, 32);  
     while (IICBSY0);                                                 /* Busy-wait until previous I2C TX is done */                 
     
     
     UART2_Send("$PMTK314,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0*28\r\n",52);    /* only output RMC messages */
          
     /* makes one 360 degree turn and calculates the x/y compensation */
     if (!P5_bit.no1)  /* Start calibration when Switch 3 is pressed */
      calibrateCompass();
     
     /* read calibration data from EEPROM */
     readCalibrationData();
}


/*
*********************************************************************************************************
*                                          readAccelerometer
*
* Description : Reads the XYZ acceleromet
*
* Arguments   : pointer to roll (X) and pitch (Y). X and Y values will be written to pointer destination
*
* Returns     : none
*
* Notes       : none
*********************************************************************************************************
*/
void readAccelerometer(signed int *roll, signed int *pitch)
{
      unsigned char addr = ACCEL_REG_DATASTART;                          /* Inform accelerometer of starting address at which to read next */
      unsigned char i2cbuf[4];              
      
      ReadIICReg(ACCEL_ADDR,addr,i2cbuf,4);                              /* read the bytes from the I2C slave */
      
      *roll = ((signed int)i2cbuf[1] << 8) | i2cbuf[0];                  /* Convert the returned x and y data bytes into signed data */
                                                                         /* from accelerometer: -270 is full vertical. 0 is full horizontal */
      *roll =  abs(*roll) / 3;                                            /* normalize to 90 if full vertical. 0 if full horizontal */
   
      *pitch = ((signed int)i2cbuf[3] << 8) | i2cbuf[2]; 
}

/*
*********************************************************************************************************
*                                          readTemp
*
* Description : reads the current temperatureof the onboard temp sensor
*
* Arguments   : pointer to temperature. temperature will be written to pointer destination
*
* Returns     : none
*
* Notes       : none
*********************************************************************************************************
*/
void readTemp(float *temp)
{
      unsigned char readData[] = {0, 0};
      ReadIICReg(TEMP_ADDR, TEMP_REG, readData, 2);	
      *temp = ((readData[0] << 8) + readData[1]) * 0.0078; /* magic number alert. taken from Renesas examples */
}


/*
*********************************************************************************************************
*                                          readLightSensor
*
* Description : reads the light sensor
*
* Arguments   : pointer to destination address
*
* Returns     : none
*
* Notes       : none
*********************************************************************************************************
*/
void readLightSensor(unsigned short *lux)
{
        unsigned char instrCode;
        unsigned char readData[] = {
            0,  // LSB
            0   // MSB
        }; 

        instrCode = LS_INSTR;        
        ReadIICReg(LS_ADDR,instrCode,readData,2);
        *lux = ((readData[1] << 8) + readData[0]); 
}

/*
*********************************************************************************************************
*                                          readCompass
*
* Description : reads the external compass connected to IIC
*
* Arguments   : pointer to yaw (heading) and raw X and raw Y (2D vector)
*
* Returns     : none
*
* Notes       : none
*********************************************************************************************************
*/
void readCompass(short *yaw, signed int *compass_x, signed int *compass_y)
{        
       float heading;

        unsigned char instrCodes[] =  {COMPASS_MODE_REGISTER, 1 };      // single measurement. 
        unsigned char readData[] = { 0,0,0,0,0,0 };        
        WriteIICBuf(COMPASS_ADDR_8BIT_WRITE,instrCodes,2);
        
        instrCodes[0] = COMPASS_STATUS_REGISTER;
        instrCodes[1] = 0;
        
        while ((readData[0] & COMPASS_STATUS_RDY) != COMPASS_STATUS_RDY)
        {          
          WriteIICBuf(COMPASS_ADDR_8BIT_WRITE,instrCodes,1);            // select status register
          ReadIICBuf(COMPASS_ADDR_8BIT_READ,readData,1);                // read status register          
        }
        
        instrCodes[0] = COMPASS_DATA_OUTPUT_X_MSB;
        instrCodes[1] = 0;
        WriteIICBuf(COMPASS_ADDR_8BIT_WRITE,instrCodes,1);              // select X register
        ReadIICBuf(COMPASS_ADDR_8BIT_READ,readData,6);                  // read y LSB             
                  
        *compass_x = readData[0] << 8 | readData[1];
        // compass_z = readData[2] << 8 | readData[3];
        *compass_y = readData[4] << 8 | readData[5];
        
        *compass_x -= calibrationData.calibrationX;
        *compass_y -= calibrationData.calibrationY;
        
        heading = atan2(*compass_y,*compass_x);
      
        /* Convert radians to degrees for readability. */
        *yaw = (int) RAD2DEG(heading) + 180;      
        
         *yaw += COMPASS_NORTH_CORRECTION;
       
        /* correction */
        if ((*yaw)   > 360)
         (*yaw) %= 360;

}

/*
*********************************************************************************************************
*                                          receiveNMEAMessage
*
* Description : Resets the NMEA receive buffer and enables the receive of a new nmea message
*
* Arguments   : none
*
* Returns     : none
*
* Notes       : This is a non blocking call. the ISR will send an OS message to the monitor task
                when a complete NMEA message is received
*********************************************************************************************************
*/
void receiveNMEAMessage()
{
      memset(nmea_cmd,0,NMEAP_MAX_SENTENCE_LENGTH);                        /* clear buffer */
      UART2_Receive((UCHAR *)nmea_cmd,NMEAP_MAX_SENTENCE_LENGTH-1);        /* receive next NMEA cmd */       
}

/*
*********************************************************************************************************
*                                          readGPS
*
* Description : This function is called when the ISR sends a OS message to the monitor task
                indicating that an NMEA message was received.
*
* Arguments   : pointer to azimut (heading/yaw) and elevation (pitch)
*
* Returns     : none
*
* Notes       : none
*********************************************************************************************************
*/
void readGPS(float *idealAzimuth, float *idealElevation)
{
   /* Check GPS on UART2 to see if there's anything to do */

    /* new NMEA command received */
    pushNmeaBuff((char *)nmea_cmd,strlen((char *)nmea_cmd));              /* push buffer to NMEA parser */
        
    
    if (rmc.valid == 1 && rmc.warn == 'A')                                /* check results. 'A' indicates valid content */
    {
      BSP_LED_Toggle(6);
      gpsLogData.latitude = rmc.latitude;
      gpsLogData.longitude = rmc.longitude;

      /* update date/time */
      dateTime.Sec   = rmc.time % 100;                                    /* Store time from GPS into RTC */
      dateTime.Min   = (((rmc.time % 10000) - (rmc.time % 100)) / 100);   /* time is stored as HHMMSS in a long type */
      dateTime.Hour  = (rmc.time / 10000);                       
      dateTime.Year  = 2000 + (rmc.date % 100);                           /* date is stored as DDMMYY in a long type */
      dateTime.Month = ((rmc.date % 10000) - (rmc.date % 100)) / 100;
      dateTime.Day   = rmc.date / 10000;
       
      /* adjust for GMT+2 */
      dateTime.Hour +=2;
      if (dateTime.Hour >= 22)
      {
        dateTime.Hour %= 24;
        dateTime.Day++;                                                   /* this will fail at the end of the month!! Please fix me */
      }
      
      ApproximateSolarCoordinates(&dateTime,DEG2RAD(gpsLogData.latitude),DEG2RAD(gpsLogData.longitude), idealAzimuth, idealElevation);
      *idealAzimuth = RAD2DEG(*idealAzimuth);
      *idealElevation = 90 - RAD2DEG(*idealElevation);
      rmc.valid = 0;                                                     /* Set to invalid so we know when a new rmc is parsed succesfully again */
      UART2_Send("$PMTK320,1*2E\r\n",15);                                /* go into powersaving mode */
    }  
    
    if (gga.valid == 1)
    {        
      gpsLogData.latitude = gga.latitude;
      gpsLogData.longitude = gga.longitude;
      gpsLogData.sats = (unsigned char) gga.satellites;
      gpsLogData.quality = gga.quality;
      gpsLogData.hdop = gga.hdop;
      gpsLogData.geoid = gga.geoid;
      gga.valid = 0;                                                     /* Set to invalid so we know when a new gga is parsed succesfully again */
    }        
}