/*
*********************************************************************************************************
*
*                                     
*                                           Monitor task
*                                        monitors temp, acc, compass                                     
*
* Filename      : monitor.c
* Version       : V1.00
* Programmer(s) : SBr
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include <includes.h>
#include <string.h>
#include <CG_macrodriver.h> 
#include <CG_userdefine.h>
#include <monitor.h>
#include <controller.h>
#include <peripherals.h>

/*********************************************************************************************************
*                                             LOCAL DEFINES
*********************************************************************************************************/

/* absolute platform values */
#define TILT_MIN_VALUE                      0         /* fully horizontal */
#define TILT_MAX_VALUE                      80        /* almost vertical */
/******************************************************************************
*  Global define
*******************************************************************************/


/*********************************************************************************************************
*                                            GLOBAL VARIABLES
********************************************************************************************************/
OS_TCB    App_MonitorTaskTCB; 
CPU_STK   App_MonitorTaskStk[APP_CFG_TASK_MONITOR_STK_SIZE];
OS_Q      MonitorQueue;

/*********************************************************************************************************
*                                            LOCAL VARIABLES
********************************************************************************************************/
/* measured values */
static signed int roll = 0;                 /* X value of accelerometer */
static signed int pitch = 0;                /* Y value of accelerometer */
static signed short yaw = 0;                  /* compass value in degrees */
static signed int compass_x = 0;            /* internal compass values. for test/calibration purposes */
static signed int compass_y = 0;            /* internal compass values. for test/calibration purposes */

// static signed int compass_z = 0;        /* internal compass values. for test/calibration purposes */
static unsigned short lux = 0;              /* Light sensor value */
static float          temp;                 /* Temp sensor */

/* calculated values */
static float idealAzimuth;              /* calculated ideal azumuth based on current UTC time and Lat/Lon position */
static float idealElevation;            /* calculated ideal elecation of solar panel based on current UTC time and Lat/Lon position */

DateTime dateTime;

/* log message buffers */
unsigned char StoreGpsLocation[64];          /* stores GPS info on SD card in format             Lon:%10f Lat:%10f Sats:%2i\n */                                            /* stores temp, light, accelerometer and compass value to SD card */
unsigned char logBuffer[64];                 /* format: |T|%.3f|L|%i|C|%i|%i|%i|X|%i|Y|%i,temp,lux,yaw,compass_x,compass_y,roll,pitch */
unsigned char timeStamp[19];

#define MAX_MESSAGES 8
ControlMessage Messages[MAX_MESSAGES];      /* message array */

OS_TMR adjustPlatformTmr;
OS_TMR periodicLoggingTmr;

/*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************/
static void logMeasurements();
static void adjustPlatform();

ControlMessage *GetAvailableMessage();

/*********************************************************************************************************
*                                         EXTERNALS
*********************************************************************************************************/

      

/*********************************************************************************************************
*                                          Monitor TASK
*
* Description : This task monitors all I2C, UART and SPI devices. Currently temperature, accelerometer, light, compass and GPS
*
* Arguments   : p_arg   is the argument passed to 'App_MonitorTask()' by 'OSTaskCreate()'.
*
* Returns     : none
*
* Notes       : 1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                  used.  The compiler should not generate any code for this statement.
*********************************************************************************************************
*/



void ResetRL78()
{
  /* execute illigal instruction to force reset */
   asm ("db 0xFF");
}

void  App_MonitorTask (void *p_arg)
{
    OS_ERR          err;    
    (void)          p_arg;
    CPU_TS          ts;
    OS_MSG_SIZE     msg_size;   
            
    
     /* init message array. reset handled flag */
    for (int i=0;i<MAX_MESSAGES;i++)
       Messages[i].MessageHandled = true;     
    
    initPeripherals();                                /* init all internal and external peripherals */

    /* Start periodic timers */    
    OSTmrCreate(&periodicLoggingTmr,
                 "periodic logging",
                 0,                                  /* no initial delay */
                 100,                                /* run every 10 sec */
                 OS_OPT_TMR_PERIODIC,            
                 logMeasurements,                     /* the adjust function */
                 0,                                  /* no arguments */
                 &err);


   OSTmrCreate(&adjustPlatformTmr,
                 "adjust Platform",
                 0,                                  /* no initial delay */
                 200,                                /* run every 20 sec */
                 OS_OPT_TMR_PERIODIC,            
                 adjustPlatform,                     /* the adjust function */
                 0,                                  /* no arguments */
                 &err);
      
    /* Start the OS timers for periodic actions */
    OSTmrStart(&periodicLoggingTmr, &err);
    OSTmrStart(&adjustPlatformTmr,  &err);

    while(DEF_TRUE)
    {                          
          // RTC_Get_CounterValue(&rtcValue);                                    /* Get current RTC time  */                                                       
          BSP_LED_Toggle(1);                                                  /* Showing that this task is still alive */
                     
          receiveNMEAMessage();                                               /* ready to receive NMEA message */
                    
          OSQPend((OS_Q   *) &MonitorQueue,                                   /* wait for next NMEA message */
                  (OS_TICK) 0,
                  (OS_OPT) OS_OPT_PEND_BLOCKING,
                  (OS_MSG_SIZE *) &msg_size,
                  (CPU_TS *) &ts,
                  (OS_ERR   *) &err);           
          
          readGPS(&idealAzimuth,&idealElevation);                                               /* parse new NMEA message */
    }
}



/* returns first available message from message array */
static ControlMessage *GetAvailableMessage()
{
   unsigned char i;
   
   for (i=0;i<MAX_MESSAGES;i++)
   {
      if (Messages[i].MessageHandled)
        return &Messages[i];
   }
   
   /* Out of messages! This Should not happen */
   ResetRL78();

   return 0; /* we'll never get here but compiler complains */
}

void SendMessage(unsigned char cmd, unsigned char *data)
{
    OS_ERR err;
    
    ControlMessage *message;
    message = GetAvailableMessage();
    
    if (message)
    {
        message->Cmd = cmd;
        message->MessageHandled = false;
        if (data)
          message->Data = data;
        
        OSQPost((OS_Q      *) &ControlQueue,
                (void      *) message,
                (OS_MSG_SIZE) sizeof(void *),   // not used by controller
                (OS_OPT     ) OS_OPT_POST_FIFO,
                (OS_ERR    *) &err);
    } 
}



static void logMeasurements()
{
      /* read sensor values */
      readAccelerometer(&roll, &pitch);
      readCompass(&yaw, &compass_x, &compass_y);  
      readLightSensor(&lux);
      readTemp(&temp);   
        
      /* do the timeStamp here because we have more stack space in this task.. */
      sprintf((char *)timeStamp,"%4i%2i%2i %2i:%2i:%2i ",dateTime.Year,dateTime.Month,dateTime.Day,dateTime.Hour,dateTime.Min,dateTime.Sec);

      sprintf((char *)logBuffer,"|T|%.3f|L|%i|C|%i|%i|%i|X|%i|Y|%i\n",temp,lux,yaw,compass_x,compass_y,roll,pitch);
      SendMessage(CMD_STORE_LOG_MESSAGE,logBuffer);
      
      if (gpsLogData.longitude > 0)
      {
        sprintf((char *)StoreGpsLocation,"Lo|%f|La|%f|S|%i|Q|%i|H|%f|G|%f\n",
                                                                              gpsLogData.longitude,
                                                                              gpsLogData.latitude,
                                                                              gpsLogData.sats, 
                                                                              gpsLogData.quality, 
                                                                              (float) gpsLogData.hdop, 
                                                                              (float) gpsLogData.geoid);
        SendMessage(CMD_STORE_GPS_LOCATION,StoreGpsLocation);
      }
}

static void adjustPlatform()
{
     int             delta;
  
     if (lux >= LUX_NIGHT_VALUE)                                          /* Check if it's usefull at all to do any adjustment */
     {
       if ((idealAzimuth > 0) && (abs ((int)idealAzimuth - yaw) > 10))
       {
              SendMessage(CMD_ROTATE,0);      
       }
               
        if (idealElevation > 0)
        {
            delta = roll - ((int)idealElevation);
              
            if (delta > 5 && (roll > TILT_MIN_VALUE))
            {
               SendMessage(CMD_MOVE_UP,0);
            }
            else if (delta < -5 && (roll < TILT_MAX_VALUE))
            {
               SendMessage(CMD_MOVE_DOWN,0);
            }
        }
     }
}


